package markov;

import java.util.AbstractMap;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;
import java.util.function.Function;

public class Builder {

    private final int prefix_length;
    private final Data data;

    public Builder(int prefix_length, Data data) {
        super();
        this.prefix_length = prefix_length;
        this.data = data;
    }

    public Collection<Map.Entry<Prefix, Decission>> random() {
        return produce(p -> nextRandom(p));
    }

    public Collection<Map.Entry<Prefix, Decission>> average() {
        return produce(p -> data.fetch(p).average());
    }

    private Collection<Decission> produce(
            Function<Prefix, Decission> producer) {
        List<Decission> result = new LinkedList<>();
        Prefix p = initPrefix();
        Decission d = new Decission(Token.START, 0, p, null);
        for (int i = 0; i < 1000; i++) {
            if (d.getToken() == Token.END)
                break;
            d = producer.apply(p);
            result.add(new AbstractMap.SimpleEntry<>(p, d));
            p = p.slide(d.getToken(), prefix_length);
        }
        return result;
    }

    private Prefix initPrefix() {
        return new Prefix(new Token[0]);
    }

    private Decission nextRandom(Prefix prefix) {
        Lookup l = data.fetch(prefix);
        if (l == null) {
            throw new NullPointerException(String.format("could not find a lookup for %s", prefix));
        }
        return l.forRandom(nextRandomNumber());
    }

    private double nextRandomNumber() {
        return ThreadLocalRandom.current().nextDouble();
    }
}
