package markov;

import java.util.Collection;
import java.util.Map.Entry;

public class Renderer {
    public static class Options {
        private final boolean propability;
        private final boolean possibilities;
        private final boolean recomends;
        private final boolean specialToken;
        private final String prefix;

        public Options(boolean propability, boolean possibilities, boolean recomends, boolean specialToken, String prefix) {
            this.propability = propability;
            this.possibilities = possibilities;
            this.recomends = recomends;
            this.specialToken = specialToken;
            this.prefix = prefix;
        }

        public static final Options NONE = new Options(false, false, false, false, "");
        public static final Options FULL = new Options(true, true, true, true, "");
        public static final Options TOKEN = new Options(true, false, false, true, "");

    }

    private final Data data;
    private final Options options;

    public Renderer(Data data) {
        super();
        this.data = data;
        this.options = Options.NONE;
    }

    public Renderer(Data data, Options options) {
        super();
        this.data = data;
        this.options = options;
    }

    public String render(Collection<Entry<Prefix, Decission>> sentence) {

        double p = 1;
        StringBuilder sb = new StringBuilder();
        if (options.recomends)
            sb.append("KAI-uwe empfiehlt");
        for (Entry<Prefix, Decission> t : sentence) {
            Lookup lookup = data.fetch(t.getKey());
            int possibilities = lookup.getDistinctTokens();
            p *= (double) lookup.getAmount(t.getValue().getToken())
                    / (double) lookup.getTotalCounts();
            if (options.possibilities) {

                if (possibilities <= 1) {
                    sb.append("-");
                } else if (possibilities <= 2) {
                    sb.append("--");
                } else if (possibilities <= 3) {
                    sb.append("---");
                } else if (possibilities <= 5) {
                    sb.append("_");
                } else if (possibilities <= 10) {
                    sb.append("__");
                } else if (possibilities <= 20) {
                    sb.append("___");
                } else if (possibilities <= 50) {
                    sb.append("*");
                } else if (possibilities <= 100) {
                    sb.append("**");
                } else if (possibilities <= 200) {
                    sb.append("***");
                } else {
                    sb.append(" ");
                }
            }
            sb.append(t.getValue().getToken().render(options.prefix));
        }
        if (options.propability) {
            sb.append(" - " + p);
        }
        return sb.toString();
    }



}
